@@ -17,14 +17,19 @@
 # along with Lava Server.  If not, see <http://www.gnu.org/licenses/>.
 
 import csv
-import simplejson
 import os
+import shutil
+import simplejson
 import tempfile
 
 from django.contrib.auth.decorators import login_required
 from django.contrib.auth.models import Group, User
 from django.core import serializers
-from django.core.exceptions import PermissionDenied, ValidationError
+from django.core.exceptions import (
+    PermissionDenied,
+    ValidationError,
+    FieldError
+)
 from django.core.urlresolvers import reverse
 from django.http import (
     HttpResponse,
@@ -32,9 +37,10 @@ from django.http import (
     HttpResponseRedirect
 )
 from django.shortcuts import get_object_or_404, render_to_response
-from django.template import RequestContext
+from django.template import RequestContext, defaultfilters
 from django.utils.safestring import mark_safe
 
+from dashboard_app.models import NamedAttribute
 from lava_server.bread_crumbs import (
     BreadCrumb,
     BreadCrumbTrail,
@@ -51,6 +57,9 @@ from lava_results_app.models import (
     Query,
     QueryGroup,
     QueryCondition,
+    QueryUpdatedError,
+    TestCase,
+    TestSuite
 )
 
 from lava_results_app.views.query.tables import (
@@ -60,7 +69,6 @@ from lava_results_app.views.query.tables import (
     QueryTestJobTable,
     QueryTestCaseTable,
     QueryTestSuiteTable,
-    QueryTestSetTable,
 )
 
 from lava_scheduler_app.models import TestJob
@@ -73,9 +81,16 @@ from django_tables2 import (
 from lava.utils.lavatable import LavaView
 
 
+CONDITIONS_SEPARATOR = ','
+CONDITION_DIVIDER = '__'
+
+
 class InvalidConditionsError(Exception):
-    ''' Raise when querying by URL has incorrect condition arguments. '''
-    pass
+    """ Raise when querying by URL has incorrect condition arguments. """
+
+
+class InvalidContentTypeError(Exception):
+    """ Raise when querying by URL content type (table name). """
 
 
 class UserQueryView(LavaView):
@@ -111,24 +126,35 @@ class GroupQueryView(LavaView):
 
 
 QUERY_CONTENT_TYPE_TABLE = {
-    "testjob": QueryTestJobTable,
-    "testcase": QueryTestCaseTable,
-    "testsuite": QueryTestSuiteTable,
-    "testset": QueryTestSetTable
+    TestJob: QueryTestJobTable,
+    TestCase: QueryTestCaseTable,
+    TestSuite: QueryTestSuiteTable,
 }
 
 
-class QueryResultView(LavaView):
+class QueryCustomResultView(LavaView):
 
     def __init__(self, content_type, conditions,
                  request, **kwargs):
         self.content_type = content_type
         self.conditions = conditions
         self.request = request
+        super(QueryCustomResultView, self).__init__(request, **kwargs)
+
+    def get_queryset(self):
+        return Query.get_queryset(self.content_type,
+                                  self.conditions).visible_by_user(user)
+
+
+class QueryResultView(LavaView):
+
+    def __init__(self, query, request, **kwargs):
+        self.query = query
+        self.request = request
         super(QueryResultView, self).__init__(request, **kwargs)
 
     def get_queryset(self):
-        return Query.get_results(self.content_type, self.conditions)
+        return self.query.get_results(self.request.user)
 
 
 @BreadCrumb("Queries", parent=index)
@@ -194,20 +220,18 @@ def query_list(request):
 @BreadCrumb("Query ~{username}/{name}", parent=query_list,
             needs=['username', 'name'])
 @login_required
-@ownership_required
 def query_display(request, username, name):
 
     query = get_object_or_404(Query, owner__username=username, name=name)
 
     view = QueryResultView(
-        content_type=query.content_type,
-        conditions=query.querycondition_set.all(),
+        query=query,
         request=request,
         model=query.content_type.model_class(),
-        table_class=QUERY_CONTENT_TYPE_TABLE[query.content_type.model]
+        table_class=QUERY_CONTENT_TYPE_TABLE[query.content_type.model_class()]
     )
 
-    table = QUERY_CONTENT_TYPE_TABLE[query.content_type.model](
+    table = QUERY_CONTENT_TYPE_TABLE[query.content_type.model_class()](
         view.get_table_data()
     )
 
@@ -218,7 +242,7 @@ def query_display(request, username, name):
         'lava_results_app/query_display.html', {
             'query': query,
             'entity': query.content_type.model,
-            'conditions': _join_conditions(query),
+            'conditions': _serialize_conditions(query),
             'query_table': table,
             'terms_data': table.prepare_terms_data(view),
             'search_data': table.prepare_search_data(view),
@@ -234,30 +258,40 @@ def query_display(request, username, name):
 @login_required
 def query_custom(request):
 
-    # TODO: validate entity as content_type
-    # TODO: validate conditions
+    content_type = _get_content_type(request.GET.get("entity"))
 
-    content_type = ContentType.objects.get(
-        model=request.GET.get("entity"),
-        app_label=_get_app_label_for_model(request.GET.get("entity"))
-    )
+    if content_type.model_class() not in QueryCondition.RELATION_MAP:
+        raise InvalidContentTypeError(
+            "Wrong table name in entity param. Please refer to query docs.")
 
-    view = QueryResultView(
+    view = QueryCustomResultView(
         content_type=content_type,
         conditions=_parse_conditions(content_type,
                                      request.GET.get("conditions")),
         request=request,
         model=content_type.model_class(),
-        table_class=QUERY_CONTENT_TYPE_TABLE[content_type.model]
+        table_class=QUERY_CONTENT_TYPE_TABLE[content_type.model_class()]
     )
 
-    table = QUERY_CONTENT_TYPE_TABLE[content_type.model](
-        view.get_table_data()
-    )
+    try:
+        table = QUERY_CONTENT_TYPE_TABLE[content_type.model_class()](
+            view.get_table_data()
+        )
+    except FieldError:
+        raise InvalidConditionsError("Conditions URL incorrect: Field does "
+                                     "not exist. Please refer to query docs.")
+
+    config = RequestConfig(request, paginate={"per_page": table.length})
+    config.configure(table)
 
     return render_to_response(
         'lava_results_app/query_custom.html', {
             'query_table': table,
+
+            'terms_data': table.prepare_terms_data(view),
+            'search_data': table.prepare_search_data(view),
+            'discrete_data': table.prepare_discrete_data(view),
+
             'bread_crumb_trail': BreadCrumbTrail.leading_to(query_custom),
             'context_help': BreadCrumbTrail.leading_to(query_list),
         }, RequestContext(request)
@@ -265,22 +299,29 @@ def query_custom(request):
 
 
 def _parse_conditions(content_type, conditions):
-    # Parse conditions from request.
+    # Parse conditions from text representation.
+
+    if not conditions:
+        return []
 
     conditions_objects = []
-    for condition_str in conditions.split(","):
+    for condition_str in conditions.split(CONDITIONS_SEPARATOR):
         condition = QueryCondition()
-        condition_fields = condition_str.split("__")
+        condition_fields = condition_str.split(CONDITION_DIVIDER)
         if len(condition_fields) == 3:
             condition.table = content_type
             condition.field = condition_fields[0]
             condition.operator = condition_fields[1]
             condition.value = condition_fields[2]
         elif len(condition_fields) == 4:
-            content_type = ContentType.objects.get(
-                model=condition_fields[0],
-                app_label=_get_app_label_for_model(condition_fields[0])
-            )
+
+            try:
+                content_type = _get_content_type(condition_fields[0])
+            except ContentType.DoesNotExist:
+                raise InvalidContentTypeError(
+                    "Wrong table name in conditions parameter. " +
+                    "Please refer to query docs.")
+
             condition.table = content_type
             condition.field = condition_fields[1]
             condition.operator = condition_fields[2]
@@ -288,40 +329,42 @@ def _parse_conditions(content_type, conditions):
 
         else:
             # TODO: more validation for conditions?.
-            raise InvalidConditionsError("Conditions URL incorrect.")
+            raise InvalidConditionsError("Conditions URL incorrect. Please "
+                                         "refer to query docs.")
 
         conditions_objects.append(condition)
 
     return conditions_objects
 
 
-def _join_conditions(query):
-    # Join conditions for query by URL.
-    conditions = None
+def _serialize_conditions(query):
+    # Serialize conditions into string.
+
+    conditions_list = []
     for condition in query.querycondition_set.all():
-        if conditions:
-            conditions += ','
-        else:
-            conditions = ''
-        conditions += '{0}__{1}__{2}__{3}'.format(
+        conditions_list.append("%s%s%s%s%s%s%s" % (
             condition.table.model,
+            CONDITION_DIVIDER,
             condition.field,
+            CONDITION_DIVIDER,
             condition.operator,
+            CONDITION_DIVIDER,
             condition.value
-        )
+        ))
 
-    return conditions
+    return CONDITIONS_SEPARATOR.join(conditions_list)
 
 
-def _get_app_label_for_model(model_name):
-    # Every model currently used is in 'lava_results_app', except the
-    # TestJob, hence the hack.
+def _get_content_type(model_name):
+    if (model_name == ContentType.objects.get_for_model(TestCase).model):
+        return ContentType.objects.get_for_model(TestCase)
 
-    app_label = Query._meta.app_label
-    if model_name == "testjob":
-        app_label = TestJob._meta.app_label
-
-    return app_label
+    content_types = ContentType.objects.filter(model=model_name)
+    if (len(content_types) == 0):
+        raise InvalidContentTypeError(
+            "Wrong table name in entity param. Please refer to query docs.")
+    else:
+        return ContentType.objects.filter(model=model_name)[0]
 
 
 @BreadCrumb("Query ~{username}/{name}", parent=query_list,
@@ -331,15 +374,17 @@ def _get_app_label_for_model(model_name):
 def query_detail(request, username, name):
 
     query = get_object_or_404(Query, owner__username=username, name=name)
+    query_conditions = _serialize_conditions(query)
 
     return render_to_response(
         'lava_results_app/query_detail.html', {
             'query': query,
+            'query_conditions': query_conditions,
             'bread_crumb_trail': BreadCrumbTrail.leading_to(
                 query_detail, username=username, name=name),
             'context_help': BreadCrumbTrail.leading_to(query_list),
             'condition_form': QueryConditionForm(
-                request.user, instance=None,
+                instance=None,
                 initial={'query': query, 'table': query.content_type}),
         }, RequestContext(request)
     )
@@ -353,10 +398,7 @@ def query_add(request):
     query.owner = request.user
 
     if request.GET.get("entity"):
-        query.content_type = ContentType.objects.get(
-            model=request.GET.get("entity"),
-            app_label=_get_app_label_for_model(request.GET.get("entity"))
-        )
+        query.content_type = _get_content_type(request.GET.get("entity"))
 
     return query_form(
         request,
@@ -389,6 +431,48 @@ def query_delete(request, username, name):
         'lava.results.query_list'))
 
 
+@login_required
+def query_export(request, username, name):
+    """
+    Create and serve the CSV file.
+    """
+    query = get_object_or_404(Query, owner__username=username, name=name)
+
+    results = query.get_results(request.user)
+    filename = "query_%s_%s_export" % (query.owner.username, query.name)
+    return _export_query(results, query.content_type, filename)
+
+
+@login_required
+def query_export_custom(request):
+    """
+    Create and serve the CSV file.
+    """
+
+    try:
+        content_type = _get_content_type(request.GET.get("entity"))
+    except ContentType.DoesNotExist:
+        raise InvalidContentTypeError(
+            "Wrong table name in entity param. Please refer to query docs.")
+
+    if content_type.model_class() not in QueryCondition.RELATION_MAP:
+        raise InvalidContentTypeError(
+            "Wrong table name in entity param. Please refer to query docs.")
+
+    conditions = _parse_conditions(content_type, request.GET.get("conditions"))
+
+    filename = "query_%s_export" % (content_type)
+
+    try:
+        results = Query.get_queryset(content_type, conditions).visible_by_user(
+            user)
+    except FieldError:
+        raise InvalidConditionsError("Conditions URL incorrect: Field does "
+                                     "not exist. Please refer to query docs.")
+
+    return _export_query(results, content_type, filename)
+
+
 @login_required
 @ownership_required
 def query_toggle_published(request, username, name):
@@ -401,6 +485,117 @@ def query_toggle_published(request, username, name):
     return HttpResponseRedirect(query.get_absolute_url() + "/+detail")
 
 
+@BreadCrumb("Copy", parent=query_detail, needs=['username', 'name'])
+@login_required
+@ownership_required
+def query_copy(request, username, name):
+    query = get_object_or_404(Query, owner__username=username, name=name)
+
+    if not request.user.is_superuser:
+        if query.owner != request.user:
+            if not query.group or not request.user.groups.filter(
+                    name=query.group.name).exists():
+                raise PermissionDenied()
+
+    return query_form(
+        request,
+        BreadCrumbTrail.leading_to(query_copy, name=name, username=username),
+        instance=query,
+        is_copy=True)
+
+
+@login_required
+def query_refresh(request, name, username):
+
+    query = get_object_or_404(Query, owner__username=username, name=name)
+
+    success = True
+    error_msg = None
+    try:
+        query.refresh_view()
+    except QueryUpdatedError as e:
+        error_msg = str(e)
+        success = False
+    except Exception as e:
+        raise
+
+    last_updated = defaultfilters.date(query.last_updated, "DATETIME_FORMAT")
+    return HttpResponse(simplejson.dumps([success, str(last_updated),
+                                          error_msg]),
+                        content_type='application/json')
+
+
+@login_required
+def query_group_list(request):
+
+    term = request.GET['term']
+    groups = [str(group.name) for group in QueryGroup.objects.filter(
+        name__istartswith=term)]
+    return HttpResponse(simplejson.dumps(groups),
+                        content_type='application/json')
+
+
+@login_required
+def query_add_group(request, username, name):
+
+    if request.method != 'POST':
+        raise PermissionDenied
+
+    group_name = request.POST.get("value")
+    query = get_object_or_404(Query, owner__username=username, name=name)
+    old_group = query.query_group
+
+    if not group_name:
+        query.query_group = None
+    else:
+        new_group = QueryGroup.objects.get_or_create(name=group_name)[0]
+        query.query_group = new_group
+
+    query.save()
+
+    if old_group:
+        if not old_group.query_set.count():
+            old_group.delete()
+
+    return HttpResponse(group_name, content_type='application/json')
+
+
+@login_required
+@ownership_required
+def query_select_group(request, username, name):
+
+    if request.method != 'POST':
+        raise PermissionDenied
+
+    group_name = request.POST.get("value")
+    query = get_object_or_404(Query, owner__username=username, name=name)
+
+    if not group_name:
+        query.group = None
+    else:
+        group = Group.objects.get(name=group_name)
+        query.group = group
+
+    query.save()
+
+    return HttpResponse(group_name, content_type='application/json')
+
+
+@login_required
+def get_group_names(request):
+
+    term = request.GET['term']
+    groups = []
+    for group in Group.objects.filter(user=request.user,
+                                      name__istartswith=term):
+        groups.append(
+            {"id": group.id,
+             "name": group.name,
+             "label": group.name})
+    return HttpResponse(simplejson.dumps(groups),
+                        content_type='application/json')
+
+
 @login_required
 @ownership_required
 def query_add_condition(request, username, name):
@@ -436,12 +631,12 @@ def query_remove_condition(request, username, name, id):
     return HttpResponseRedirect(query.get_absolute_url() + "/+detail")
 
 
-def query_form(request, bread_crumb_trail, instance=None):
+def query_form(request, bread_crumb_trail, instance=None, is_copy=False):
 
     if request.method == 'POST':
 
         form = QueryForm(request.user, request.POST,
-                         instance=instance)
+                         instance=instance, is_copy=is_copy)
         if form.is_valid():
             query = form.save()
             if request.GET.get("conditions"):
@@ -454,12 +649,19 @@ def query_form(request, bread_crumb_trail, instance=None):
             return HttpResponseRedirect(query.get_absolute_url() + "/+detail")
 
     else:
-        form = QueryForm(request.user, instance=instance)
+        form = QueryForm(request.user, instance=instance, is_copy=is_copy)
         form.fields['owner'].initial = request.user
 
+    query_name = None
+    if is_copy:
+        query_name = instance.name
+        instance.name = None
+
     return render_to_response(
         'lava_results_app/query_form.html', {
             'bread_crumb_trail': bread_crumb_trail,
+            'is_copy': is_copy,
+            'query_name': query_name,
             'form': form,
         }, RequestContext(request))
 
@@ -467,7 +669,7 @@ def query_form(request, bread_crumb_trail, instance=None):
 def query_condition_form(request, query,
                          bread_crumb_trail, instance=None):
 
-    form = QueryConditionForm(request.user, request.POST, instance=instance)
+    form = QueryConditionForm(request.POST, instance=instance)
 
     if form.is_valid():
         query_condition = form.save()
@@ -479,3 +681,58 @@ def query_condition_form(request, query,
     else:
         return HttpResponse(simplejson.dumps(["fail", form.errors]),
                             content_type='application/json')
+
+
+def _remove_dir(path):
+    """ Removes directory @path. Doesn't raise exceptions. """
+    try:
+        # Delete directory.
+        shutil.rmtree(path)
+    except OSError as exception:
+        # Silent exception whatever happens. If it's unexisting dir, we don't
+        # care.
+        pass
+
+
+def _export_query(query_results, content_type, filename):
+
+    tmp_dir = tempfile.mkdtemp()
+    file_path = os.path.join(tmp_dir, "%s.csv" % filename)
+
+    query_keys = content_type.model_class()._meta.get_all_field_names()
+
+    query_keys.sort()
+    # Remove non-relevant columns for CSV file.
+    removed_fields = [
+        # TestJob fields:
+        "_results_bundle", "_results_bundle_id", "_results_link", "user_id",
+        "actual_device_id", "definition", "group_id", "id",
+        "original_definition", "requested_device_id", "sub_id", "submit_token",
+        "submit_token_id", "submitter_id", "testdata", "testsuite",
+        # TestSuite fields:
+        "job_id",
+        # TestCase fields:
+        "actionlevels", "suite_id", "test_set_id"
+    ]
+    for field in removed_fields:
+        if field in query_keys:
+            query_keys.remove(field)
+
+    with open(file_path, 'wb+') as csv_file:
+        out = csv.DictWriter(csv_file, quoting=csv.QUOTE_ALL,
+                             extrasaction='ignore',
+                             fieldnames=query_keys)
+        out.writeheader()
+
+        for result in query_results:
+
+            result_dict = result.__dict__.copy()
+            out.writerow(result_dict)
+
+    response = HttpResponse(content_type='text/csv')
+    response['Content-Disposition'] = "attachment; filename=%s.csv" % filename
+    with open(file_path, 'r') as csv_file:
+        response.write(csv_file.read())
+
+    _remove_dir(tmp_dir)
+    return response
