@@ -192,8 +192,8 @@ class Settings(object):
         dbname = config.LAVA_DB_NAME if hasattr(config, 'LAVA_DB_NAME') else ''
         dbuser = config.LAVA_DB_USER if hasattr(config, 'LAVA_DB_USER') else ''
         dbpass = config.LAVA_DB_PASSWORD if hasattr(config, 'LAVA_DB_PASSWORD') else ''
-        dbhost = config.LAVA_DB_SERVER if (hasattr(config, 'LAVA_DB_SERVER')
-                                           and config.LAVA_DB_SERVER is not "") else '127.0.0.1'
+        dbhost = config.LAVA_DB_SERVER if (hasattr(config, 'LAVA_DB_SERVER') and
+                                           config.LAVA_DB_SERVER is not "") else '127.0.0.1'
         dbport = config.LAVA_DB_PORT if hasattr(config, 'LAVA_DB_PORT') else ''
         return {
             'ENGINE': pgengine,
